@@ -13,7 +13,7 @@
 #include "vim.h"
 #include "termlib.pro"
 
-#if !defined(AMIGA) && !defined(VMS) && !defined(MACOS) && !defined(RISCOS)
+#if !defined(AMIGA) && !defined(VMS) && !defined(MACOS)
 # include <sgtty.h>
 #endif
 
@@ -191,7 +191,7 @@ nextent(tbuf, termcap, buflen)		/* Read 1 entry from TERMCAP file */
 	    lbuf[0] == '\t' &&
 	    lbuf[1] == ':')
 	{
-	    mch_memmove(lbuf, lbuf + 2, strlen(lbuf + 2) + 1);
+	    STRMOVE(lbuf, lbuf + 2);
 	    llen -= 2;
 	}
 	if (lbuf[llen-2] == '\\')		/* and continuations */
